'use strict';

function _toArray(arr) { return Array.isArray(arr) ? arr : Array.from(arr); }

var _require = require('./helpers'),
    getReplacement = _require.getReplacement,
    replace = _require.replace,
    resolveModuleToString = _require.resolveModuleToString,
    stringToAST = _require.stringToAST,
    isCodegenComment = _require.isCodegenComment,
    isPropertyCall = _require.isPropertyCall;

module.exports = {
  asTag,
  asJSX,
  asFunction,
  asProgram,
  asImportCall,
  asImportDeclaration,
  asIdentifier
};

function asProgram(transformFromAst, path, filename) {
  var _transformFromAst = transformFromAst(path.node),
      string = _transformFromAst.code;

  var replacement = getReplacement({ string, filename });
  path.node.body = Array.isArray(replacement) ? replacement : [replacement];
}

function asImportDeclaration(path, filename) {
  var codegenComment = path.node.source.leadingComments.find(isCodegenComment).value.trim();
  var args = void 0;
  if (codegenComment !== 'codegen') {
    args = codegenComment.replace(/codegen\((.*)\)/, '$1').trim();
  }

  replace({
    path,
    string: `
      try {
        // allow for transpilation of required modules
        require('babel-register')
      } catch (e) {
        // ignore error
      }
      var mod = require('${path.node.source.value}');
      mod = mod && mod.__esModule ? mod.default : mod
      ${args ? `mod = mod(${args})` : ''}
      module.exports = mod
    `,
    filename
  });
}

function asIdentifier(path, filename) {
  var targetPath = path.parentPath;
  switch (targetPath.type) {
    case 'TaggedTemplateExpression':
      {
        return asTag(targetPath, filename);
      }
    case 'CallExpression':
      {
        var isCallee = targetPath.get('callee') === path;
        if (isCallee) {
          return asFunction(targetPath, filename);
        } else {
          return false;
        }
      }
    case 'JSXOpeningElement':
      {
        var jsxElement = targetPath.parentPath;
        return asJSX(jsxElement, filename);
      }
    case 'JSXClosingElement':
      {
        // ignore the closing element
        // but don't mark as unhandled (return false)
        // we already handled the opening element
        return true;
      }
    case 'MemberExpression':
      {
        var callPath = targetPath.parentPath;
        var isRequireCall = isPropertyCall(callPath, 'require');
        if (isRequireCall) {
          return asImportCall(callPath, filename);
        } else {
          return false;
        }
      }
    default:
      {
        return false;
      }
  }
}

function asImportCall(path, filename) {
  var _path$get = path.get('arguments'),
      _path$get2 = _toArray(_path$get),
      source = _path$get2[0],
      args = _path$get2.slice(1);

  var string = resolveModuleToString({ args, filename, source });
  var replacement = stringToAST(string);
  if (!replacement) {
    path.remove();
  } else if (Array.isArray(replacement)) {
    path.replaceWithMultiple(replacement);
  } else {
    path.replaceWith(replacement);
  }
}

function asTag(path, filename) {
  var string = path.get('quasi').evaluate().value;
  if (!string) {
    throw path.buildCodeFrameError('Unable to determine the value of your codegen string', Error);
  }
  replace({ path, string, filename });
}

function asFunction(path, filename) {
  var argumentsPaths = path.get('arguments');
  var string = argumentsPaths[0].evaluate().value;
  replace({
    path: argumentsPaths[0].parentPath,
    string,
    filename
  });
}

function asJSX(path, filename) {
  var children = path.get('children');
  var string = children[0].node.expression.value;
  if (children[0].node.expression.type === 'TemplateLiteral') {
    string = children[0].get('expression').evaluate().value;
  }
  replace({
    path: children[0].parentPath,
    string,
    filename
  });
}

/*
eslint
  complexity: ["error", 8]
*/